@@ -244,14 +244,14 @@ theora_dec_stop (GstVideoDecoder * decoder)
   th_decode_free (dec->decoder);
   dec->decoder = NULL;
   gst_theora_dec_reset (dec);
-  if (dec->tags) {
-    gst_tag_list_free (dec->tags);
-    dec->tags = NULL;
-  }
-  if (dec->input_state)
+  if (dec->input_state) {
     gst_video_codec_state_unref (dec->input_state);
-  if (dec->output_state)
+    dec->input_state = NULL;
+  }
+  if (dec->output_state) {
     gst_video_codec_state_unref (dec->output_state);
+    dec->output_state = NULL;
+  }
 
   return TRUE;
 }
@@ -380,9 +380,10 @@ theora_handle_comment_packet (GstTheoraDec * dec, ogg_packet * packet)
         GST_TAG_NOMINAL_BITRATE, dec->info.target_bitrate, NULL);
   }
 
-  if (dec->tags)
-    gst_tag_list_free (dec->tags);
-  dec->tags = list;
+  gst_video_decoder_merge_tags (GST_VIDEO_DECODER (dec),
+      list, GST_TAG_MERGE_REPLACE);
+
+  gst_tag_list_unref (list);
 
   return GST_FLOW_OK;
 }
@@ -504,15 +505,9 @@ theora_handle_type_packet (GstTheoraDec * dec)
       break;
   }
 
-  dec->have_header = TRUE;
+  gst_video_decoder_negotiate (GST_VIDEO_DECODER (dec));
 
-  /* FIXME : Put this on the next outgoing frame */
-  /* FIXME :  */
-  if (dec->tags) {
-    gst_pad_push_event (GST_VIDEO_DECODER (dec)->srcpad,
-        gst_event_new_tag ("GstDecoder", dec->tags));
-    dec->tags = NULL;
-  }
+  dec->have_header = TRUE;
 
   return ret;
 
@@ -576,7 +571,7 @@ theora_handle_image (GstTheoraDec * dec, th_ycbcr_buffer buf,
   gint pic_width, pic_height;
   gint offset_x, offset_y;
 
-  result = gst_video_decoder_alloc_output_frame (decoder, frame);
+  result = gst_video_decoder_allocate_output_frame (decoder, frame);
 
   if (G_UNLIKELY (result != GST_FLOW_OK)) {
     GST_DEBUG_OBJECT (dec, "could not get buffer, reason: %s",
@@ -733,7 +728,7 @@ not_initialized:
 dropping:
   {
     GST_WARNING_OBJECT (dec, "dropping frame because we need a keyframe");
-    return GST_VIDEO_DECODER_FLOW_NEED_DATA;
+    return GST_CUSTOM_FLOW_DROP;
   }
 dropping_qos:
   {
@@ -787,6 +782,7 @@ theora_dec_decode_buffer (GstTheoraDec * dec, GstBuffer * buf,
   if (packet.bytes && packet.packet[0] & 0x80) {
     if (dec->have_header) {
       GST_WARNING_OBJECT (GST_OBJECT (dec), "Ignoring header");
+      result = GST_CUSTOM_FLOW_DROP;
       goto done;
     }
     result = theora_handle_header_packet (dec, &packet);
@@ -848,11 +844,12 @@ theora_dec_decide_allocation (GstVideoDecoder * decoder, GstQuery * query)
 
   dec->can_crop = FALSE;
   config = gst_buffer_pool_get_config (pool);
-  if (gst_query_has_allocation_meta (query, GST_VIDEO_META_API_TYPE)) {
+  if (gst_query_find_allocation_meta (query, GST_VIDEO_META_API_TYPE, NULL)) {
     gst_buffer_pool_config_add_option (config,
         GST_BUFFER_POOL_OPTION_VIDEO_META);
     dec->can_crop =
-        gst_query_has_allocation_meta (query, GST_VIDEO_CROP_META_API_TYPE);
+        gst_query_find_allocation_meta (query, GST_VIDEO_CROP_META_API_TYPE,
+        NULL);
   }
 
   if (dec->can_crop) {
