@@ -93,7 +93,8 @@ typedef struct _GstRTPHeader
  * Allocate enough data in @buffer to hold an RTP packet with @csrc_count CSRCs,
  * a payload length of @payload_len and padding of @pad_len.
  * @buffer must be writable and all previous memory in @buffer will be freed.
- * All other RTP header fields will be set to 0/FALSE.
+ * If @pad_len is >0, the padding bit will be set. All other RTP header fields
+ * will be set to 0/FALSE.
  */
 void
 gst_rtp_buffer_allocate_data (GstBuffer * buffer, guint payload_len,
@@ -101,23 +102,26 @@ gst_rtp_buffer_allocate_data (GstBuffer * buffer, guint payload_len,
 {
   GstMapInfo map;
   GstMemory *mem;
-  gsize len;
+  gsize hlen;
 
   g_return_if_fail (csrc_count <= 15);
   g_return_if_fail (GST_IS_BUFFER (buffer));
+  g_return_if_fail (pad_len <= 255);
   g_return_if_fail (gst_buffer_is_writable (buffer));
 
   gst_buffer_remove_all_memory (buffer);
 
-  len = GST_RTP_HEADER_LEN + csrc_count * sizeof (guint32)
-      + payload_len + pad_len;
+  hlen = GST_RTP_HEADER_LEN + csrc_count * sizeof (guint32);
 
-  mem = gst_allocator_alloc (NULL, len, NULL);
+  mem = gst_allocator_alloc (NULL, hlen, NULL);
 
   gst_memory_map (mem, &map, GST_MAP_WRITE);
   /* fill in defaults */
   GST_RTP_HEADER_VERSION (map.data) = GST_RTP_VERSION;
-  GST_RTP_HEADER_PADDING (map.data) = FALSE;
+  if (pad_len)
+    GST_RTP_HEADER_PADDING (map.data) = TRUE;
+  else
+    GST_RTP_HEADER_PADDING (map.data) = FALSE;
   GST_RTP_HEADER_EXTENSION (map.data) = FALSE;
   GST_RTP_HEADER_CSRC_COUNT (map.data) = csrc_count;
   memset (GST_RTP_HEADER_CSRC_LIST_OFFSET (map.data, 0), 0,
@@ -130,11 +134,26 @@ gst_rtp_buffer_allocate_data (GstBuffer * buffer, guint payload_len,
   gst_memory_unmap (mem, &map);
 
   gst_buffer_append_memory (buffer, mem);
+
+  if (payload_len) {
+    mem = gst_allocator_alloc (NULL, payload_len, NULL);
+    gst_buffer_append_memory (buffer, mem);
+  }
+  if (pad_len) {
+    mem = gst_allocator_alloc (NULL, pad_len, NULL);
+
+    gst_memory_map (mem, &map, GST_MAP_WRITE);
+    map.data[pad_len - 1] = pad_len;
+    gst_memory_unmap (mem, &map);
+
+    gst_buffer_append_memory (buffer, mem);
+  }
 }
 
 /**
  * gst_rtp_buffer_new_take_data:
- * @data: data for the new buffer
+ * @data: (array length=len) (transfer full) (element-type guint8):
+ *   data for the new buffer
  * @len: the length of data
  *
  * Create a new buffer and set the data and size of the buffer to @data and @len
@@ -154,7 +173,8 @@ gst_rtp_buffer_new_take_data (gpointer data, gsize len)
 
 /**
  * gst_rtp_buffer_new_copy_data:
- * @data: data for the new buffer
+ * @data: (array length=len) (element-type guint8): data for the new
+ *   buffer
  * @len: the length of data
  *
  * Create a new buffer and set the data to a copy of @len
@@ -281,29 +301,42 @@ gst_rtp_buffer_calc_payload_len (guint packet_len, guint8 pad_len,
       - pad_len;
 }
 
-/*
- * validate_data:
- * @data: the data to validate
- * @len: the length of @data to validate
- * @payload: the payload if @data represents the header only
- * @payload_len: the len of the payload
+/**
+ * gst_rtp_buffer_map:
+ * @buffer: a #GstBuffer
+ * @flags: #GstMapFlags
+ * @rtp: (out): a #GstRTPBuffer
  *
- * Checks if @data is a valid RTP packet.
+ * Map the contents of @buffer into @rtp.
  *
- * Returns: TRUE if @data is a valid RTP packet
+ * Returns: %TRUE if @buffer could be mapped.
  */
-static gboolean
-validate_data (guint8 * data, guint len, guint8 * payload, guint payload_len)
+gboolean
+gst_rtp_buffer_map (GstBuffer * buffer, GstMapFlags flags, GstRTPBuffer * rtp)
 {
   guint8 padding;
   guint8 csrc_count;
   guint header_len;
   guint8 version;
+  guint8 *data;
+  guint size;
+  gsize bufsize, skip;
+  guint idx, length;
 
-  g_return_val_if_fail (data != NULL, FALSE);
+  g_return_val_if_fail (GST_IS_BUFFER (buffer), FALSE);
+  g_return_val_if_fail (rtp != NULL, FALSE);
+  g_return_val_if_fail (rtp->buffer == NULL, FALSE);
 
+  /* map first memory, this should be the header */
+  if (!gst_buffer_map_range (buffer, 0, 1, &rtp->map[0], flags))
+    goto map_failed;
+
+  data = rtp->data[0] = rtp->map[0].data;
+  size = rtp->map[0].size;
+
+  /* the header must be completely in the first buffer */
   header_len = GST_RTP_HEADER_LEN;
-  if (G_UNLIKELY (len < header_len))
+  if (G_UNLIKELY (size < header_len))
     goto wrong_length;
 
   /* check version */
@@ -315,47 +348,86 @@ validate_data (guint8 * data, guint len, guint8 * payload, guint payload_len)
   csrc_count = (data[0] & 0x0f);
   header_len += csrc_count * sizeof (guint32);
 
+  rtp->size[0] = header_len;
+
+  bufsize = gst_buffer_get_size (buffer);
+
   /* calc extension length when present. */
   if (data[0] & 0x10) {
-    guint8 *extpos;
+    guint8 *extdata;
     guint16 extlen;
 
-    /* this points to the extension bits and header length */
-    extpos = &data[header_len];
-
-    /* skip the header and check that we have enough space */
-    header_len += 4;
-    if (G_UNLIKELY (len < header_len))
+    /* find memory for the extension bits, we find the block for the first 4
+     * bytes, all other extension bytes should also be in this block */
+    if (!gst_buffer_find_memory (buffer, header_len, 4, &idx, &length, &skip))
       goto wrong_length;
 
-    /* skip id */
-    extpos += 2;
-    /* read length as the number of 32 bits words */
-    extlen = GST_READ_UINT16_BE (extpos);
+    if (!gst_buffer_map_range (buffer, idx, length, &rtp->map[1], flags))
+      goto map_failed;
 
-    header_len += extlen * sizeof (guint32);
+    extdata = rtp->data[1] = rtp->map[1].data + skip;
+    /* skip id */
+    extdata += 2;
+    /* read length as the number of 32 bits words */
+    extlen = GST_READ_UINT16_BE (extdata);
+    extlen *= sizeof (guint32);
+    /* add id and length */
+    extlen += 4;
+
+    /* all extension bytes must be in this block */
+    if (G_UNLIKELY (rtp->map[1].size < extlen))
+      goto wrong_length;
+
+    rtp->size[1] = extlen;
+
+    header_len += rtp->size[1];
+  } else {
+    rtp->data[1] = NULL;
+    rtp->size[1] = 0;
   }
 
   /* check for padding */
   if (data[0] & 0x20) {
-    if (payload)
-      padding = payload[payload_len - 1];
-    else
-      padding = data[len - 1];
+    /* find memory for the padding bits */
+    if (!gst_buffer_find_memory (buffer, bufsize - 1, 1, &idx, &length, &skip))
+      goto wrong_length;
+
+    if (!gst_buffer_map_range (buffer, idx, length, &rtp->map[3], flags))
+      goto map_failed;
+
+    padding = rtp->map[3].data[skip];
+    if (skip + 1 < padding)
+      goto wrong_length;
+
+    rtp->data[3] = rtp->map[3].data + skip + 1 - padding;
+    rtp->size[3] = padding;
   } else {
+    rtp->data[3] = NULL;
+    rtp->size[3] = 0;
     padding = 0;
   }
 
   /* check if padding and header not bigger than packet length */
-  if (G_UNLIKELY (len < padding + header_len))
+  if (G_UNLIKELY (bufsize < padding + header_len))
     goto wrong_padding;
 
+  rtp->buffer = buffer;
+  /* we have not yet mapped the payload */
+  rtp->data[2] = NULL;
+  rtp->size[2] = 0;
+  rtp->state = 0;
+
   return TRUE;
 
   /* ERRORS */
+map_failed:
+  {
+    GST_ERROR ("failed to map memory");
+    return FALSE;
+  }
 wrong_length:
   {
-    GST_DEBUG ("len < header_len check failed (%d < %d)", len, header_len);
+    GST_DEBUG ("length check failed");
     goto dump_packet;
   }
 wrong_version:
@@ -365,88 +437,43 @@ wrong_version:
   }
 wrong_padding:
   {
-    GST_DEBUG ("padding check failed (%d - %d < %d)", len, header_len, padding);
+    GST_DEBUG ("padding check failed (%" G_GSIZE_FORMAT " - %d < %d)", bufsize,
+        header_len, padding);
     goto dump_packet;
   }
 dump_packet:
   {
-    GST_MEMDUMP ("buffer", data, len);
+    gint i;
+
+    GST_MEMDUMP ("buffer", data, size);
+
+    for (i = 0; i < G_N_ELEMENTS (rtp->map); ++i) {
+      if (rtp->data[i] != NULL)
+        gst_buffer_unmap (buffer, &rtp->map[i]);
+    }
     return FALSE;
   }
 }
 
 /**
- * gst_rtp_buffer_validate_data:
- * @data: the data to validate
- * @len: the length of @data to validate
+ * gst_rtp_buffer_unmap:
+ * @rtp: a #GstRTPBuffer
  *
- * Check if the @data and @size point to the data of a valid RTP packet.
- * This function checks the length, version and padding of the packet data.
- * Use this function to validate a packet before using the other functions in
- * this module.
- *
- * Returns: TRUE if the data points to a valid RTP packet.
+ * Unmap @rtp previously mapped with gst_rtp_buffer_map().
  */
-gboolean
-gst_rtp_buffer_validate_data (guint8 * data, gsize len)
-{
-  return validate_data (data, len, NULL, 0);
-}
-
-/**
- * gst_rtp_buffer_validate:
- * @buffer: the buffer to validate
- *
- * Check if the data pointed to by @buffer is a valid RTP packet using
- * gst_rtp_buffer_validate_data().
- * Use this function to validate a packet before using the other functions in
- * this module.
- *
- * Returns: TRUE if @buffer is a valid RTP packet.
- */
-gboolean
-gst_rtp_buffer_validate (GstBuffer * buffer)
-{
-  gboolean res;
-  GstMapInfo map;
-
-  g_return_val_if_fail (GST_IS_BUFFER (buffer), FALSE);
-
-  gst_buffer_map (buffer, &map, GST_MAP_READ);
-  res = validate_data (map.data, map.size, NULL, 0);
-  gst_buffer_unmap (buffer, &map);
-
-  return res;
-}
-
-gboolean
-gst_rtp_buffer_map (GstBuffer * buffer, GstMapFlags flags, GstRTPBuffer * rtp)
-{
-  g_return_val_if_fail (GST_IS_BUFFER (buffer), FALSE);
-  g_return_val_if_fail (rtp != NULL, FALSE);
-  g_return_val_if_fail (rtp->buffer == NULL, FALSE);
-
-  if (!gst_buffer_map (buffer, &rtp->map[0], flags))
-    return FALSE;
-
-  rtp->buffer = buffer;
-  rtp->state = 0;
-  rtp->n_map = 1;
-
-  return TRUE;
-}
-
-gboolean
+void
 gst_rtp_buffer_unmap (GstRTPBuffer * rtp)
 {
-  g_return_val_if_fail (rtp != NULL, FALSE);
-  g_return_val_if_fail (rtp->buffer != NULL, FALSE);
+  gint i;
 
-  gst_buffer_unmap (rtp->buffer, &rtp->map[0]);
+  g_return_if_fail (rtp != NULL);
+  g_return_if_fail (rtp->buffer != NULL);
+
+  for (i = 0; i < 4; i++) {
+    if (rtp->data[i])
+      gst_buffer_unmap (rtp->buffer, &rtp->map[i]);
+  }
   rtp->buffer = NULL;
-  rtp->n_map = 0;
-
-  return TRUE;
 }
 
 
@@ -463,7 +490,9 @@ gst_rtp_buffer_set_packet_len (GstRTPBuffer * rtp, guint len)
 {
   guint8 *data;
 
-  data = rtp->map[0].data;
+  data = rtp->data[0];
+
+  /* FIXME */
 
   if (rtp->map[0].maxsize <= len) {
     /* FIXME, realloc bigger space */
@@ -503,16 +532,7 @@ gst_rtp_buffer_get_packet_len (GstRTPBuffer * rtp)
 guint
 gst_rtp_buffer_get_header_len (GstRTPBuffer * rtp)
 {
-  guint len;
-  guint8 *data;
-
-  data = rtp->map[0].data;
-
-  len = GST_RTP_HEADER_LEN + GST_RTP_HEADER_CSRC_SIZE (data);
-  if (GST_RTP_HEADER_EXTENSION (data))
-    len += GST_READ_UINT16_BE (data + len + 2) * 4 + 4;
-
-  return len;
+  return rtp->size[0] + rtp->size[1];
 }
 
 /**
@@ -526,7 +546,7 @@ gst_rtp_buffer_get_header_len (GstRTPBuffer * rtp)
 guint8
 gst_rtp_buffer_get_version (GstRTPBuffer * rtp)
 {
-  return GST_RTP_HEADER_VERSION (rtp->map[0].data);
+  return GST_RTP_HEADER_VERSION (rtp->data[0]);
 }
 
 /**
@@ -541,7 +561,7 @@ gst_rtp_buffer_set_version (GstRTPBuffer * rtp, guint8 version)
 {
   g_return_if_fail (version < 0x04);
 
-  GST_RTP_HEADER_VERSION (rtp->map[0].data) = version;
+  GST_RTP_HEADER_VERSION (rtp->data[0]) = version;
 }
 
 /**
@@ -555,7 +575,7 @@ gst_rtp_buffer_set_version (GstRTPBuffer * rtp, guint8 version)
 gboolean
 gst_rtp_buffer_get_padding (GstRTPBuffer * rtp)
 {
-  return GST_RTP_HEADER_PADDING (rtp->map[0].data);
+  return GST_RTP_HEADER_PADDING (rtp->data[0]);
 }
 
 /**
@@ -568,7 +588,7 @@ gst_rtp_buffer_get_padding (GstRTPBuffer * rtp)
 void
 gst_rtp_buffer_set_padding (GstRTPBuffer * rtp, gboolean padding)
 {
-  GST_RTP_HEADER_PADDING (rtp->map[0].data) = padding;
+  GST_RTP_HEADER_PADDING (rtp->data[0]) = padding;
 }
 
 /**
@@ -586,7 +606,7 @@ gst_rtp_buffer_pad_to (GstRTPBuffer * rtp, guint len)
 {
   guint8 *data;
 
-  data = rtp->map[0].data;
+  data = rtp->data[0];
 
   if (len > 0)
     GST_RTP_HEADER_PADDING (data) = TRUE;
@@ -607,7 +627,7 @@ gst_rtp_buffer_pad_to (GstRTPBuffer * rtp, guint len)
 gboolean
 gst_rtp_buffer_get_extension (GstRTPBuffer * rtp)
 {
-  return GST_RTP_HEADER_EXTENSION (rtp->map[0].data);
+  return GST_RTP_HEADER_EXTENSION (rtp->data[0]);
 }
 
 /**
@@ -620,15 +640,15 @@ gst_rtp_buffer_get_extension (GstRTPBuffer * rtp)
 void
 gst_rtp_buffer_set_extension (GstRTPBuffer * rtp, gboolean extension)
 {
-  GST_RTP_HEADER_EXTENSION (rtp->map[0].data) = extension;
+  GST_RTP_HEADER_EXTENSION (rtp->data[0]) = extension;
 }
 
 /**
  * gst_rtp_buffer_get_extension_data:
  * @rtp: the RTP packet
- * @bits: location for result bits
- * @data: location for data
- * @wordlen: location for length of @data in 32 bits words
+ * @bits: (out): location for result bits
+ * @data: (out) (array) (element-type guint8) (transfer none): location for data
+ * @wordlen: (out): location for length of @data in 32 bits words
  *
  * Get the extension data. @bits will contain the extension 16 bits of custom
  * data. @data will point to the data in the extension and @wordlen will contain
@@ -638,24 +658,17 @@ gst_rtp_buffer_set_extension (GstRTPBuffer * rtp, gboolean extension)
  * with @bits, @data and @wordlen unchanged.
  * 
  * Returns: TRUE if @buffer had the extension bit set.
- *
- * Since: 0.10.15
  */
 gboolean
 gst_rtp_buffer_get_extension_data (GstRTPBuffer * rtp, guint16 * bits,
     gpointer * data, guint * wordlen)
 {
-  guint len;
   guint8 *pdata;
 
-  pdata = rtp->map[0].data;
-
-  if (!GST_RTP_HEADER_EXTENSION (pdata))
-    return FALSE;
-
   /* move to the extension */
-  len = GST_RTP_HEADER_LEN + GST_RTP_HEADER_CSRC_SIZE (pdata);
-  pdata += len;
+  pdata = rtp->data[1];
+  if (!pdata)
+    return FALSE;
 
   if (bits)
     *bits = GST_READ_UINT16_BE (pdata);
@@ -668,6 +681,43 @@ gst_rtp_buffer_get_extension_data (GstRTPBuffer * rtp, guint16 * bits,
   return TRUE;
 }
 
+/* ensure header, payload and padding are in separate buffers */
+static void
+ensure_buffers (GstRTPBuffer * rtp)
+{
+  guint i, pos;
+  gsize offset;
+  gboolean changed = FALSE;
+
+  /* make sure payload is mapped */
+  gst_rtp_buffer_get_payload (rtp);
+
+  for (i = 0, pos = 0; i < 4; i++) {
+    if (rtp->size[i]) {
+      offset = rtp->map[i].data - (guint8 *) rtp->data[i];
+
+      if (offset != 0 || rtp->map[i].size != rtp->size[i]) {
+        GstMemory *mem;
+
+        /* make copy */
+        mem = gst_memory_copy (rtp->map[i].memory, offset, rtp->size[i]);
+
+        /* insert new memory */
+        gst_buffer_insert_memory (rtp->buffer, pos, mem);
+
+        changed = TRUE;
+      }
+      pos++;
+    }
+  }
+
+  if (changed) {
+    gst_rtp_buffer_unmap (rtp);
+    gst_buffer_remove_memory_range (rtp->buffer, pos, -1);
+    gst_rtp_buffer_map (rtp->buffer, GST_MAP_READWRITE, rtp);
+  }
+}
+
 /**
  * gst_rtp_buffer_set_extension_data:
  * @rtp: the RTP packet
@@ -676,12 +726,10 @@ gst_rtp_buffer_get_extension_data (GstRTPBuffer * rtp, guint16 * bits,
  * the extension, excluding the extension header ( therefore zero is a valid length)
  *
  * Set the extension bit of the rtp buffer and fill in the @bits and @length of the
- * extension header. It will refuse to set the extension data if the buffer is not
- * large enough.
+ * extension header. If the existing extension data is not large enough, it will
+ * be made larger.
  *
  * Returns: True if done.
- *
- * Since: 0.10.18
  */
 gboolean
 gst_rtp_buffer_set_extension_data (GstRTPBuffer * rtp, guint16 bits,
@@ -689,33 +737,50 @@ gst_rtp_buffer_set_extension_data (GstRTPBuffer * rtp, guint16 bits,
 {
   guint32 min_size = 0;
   guint8 *data;
+  GstMemory *mem = NULL;
 
-  data = rtp->map[0].data;
+  ensure_buffers (rtp);
 
-  /* check if the buffer is big enough to hold the extension */
-  min_size =
-      GST_RTP_HEADER_LEN + GST_RTP_HEADER_CSRC_SIZE (data) + 4 +
-      length * sizeof (guint32);
-  if (G_UNLIKELY (min_size > rtp->map[0].size))
-    goto too_small;
+  /* this is the size of the extension data we need */
+  min_size = 4 + length * sizeof (guint32);
+
+  /* we should allocate and map the extension data */
+  if (rtp->data[1] == NULL || min_size > rtp->size[1]) {
+    GstMapInfo map;
+
+    /* we don't have (enough) extension data, make some */
+    mem = gst_allocator_alloc (NULL, min_size, NULL);
+
+    if (rtp->data[1]) {
+      /* copy old data */
+      gst_memory_map (mem, &map, GST_MAP_WRITE);
+      memcpy (map.data, rtp->data[1], rtp->size[1]);
+      gst_memory_unmap (mem, &map);
+
+      /* unmap old */
+      gst_buffer_unmap (rtp->buffer, &rtp->map[1]);
+      gst_buffer_replace_memory (rtp->buffer, 1, mem);
+    } else {
+      /* we didn't have extension data, add */
+      gst_buffer_insert_memory (rtp->buffer, 1, mem);
+    }
+
+    /* map new */
+    gst_memory_map (mem, &rtp->map[1], GST_MAP_READWRITE);
+    gst_memory_ref (mem);
+    rtp->data[1] = rtp->map[1].data;
+    rtp->size[1] = rtp->map[1].size;
+  }
 
   /* now we can set the extension bit */
-  GST_RTP_HEADER_EXTENSION (rtp->map[0].data) = TRUE;
+  data = rtp->data[0];
+  GST_RTP_HEADER_EXTENSION (data) = TRUE;
 
-  data += GST_RTP_HEADER_LEN + GST_RTP_HEADER_CSRC_SIZE (data);
+  data = rtp->data[1];
   GST_WRITE_UINT16_BE (data, bits);
   GST_WRITE_UINT16_BE (data + 2, length);
 
   return TRUE;
-
-  /* ERRORS */
-too_small:
-  {
-    g_warning
-        ("rtp buffer too small: need more than %d bytes but only have %"
-        G_GSIZE_FORMAT " bytes", min_size, rtp->map[0].size);
-    return FALSE;
-  }
 }
 
 /**
@@ -729,7 +794,7 @@ too_small:
 guint32
 gst_rtp_buffer_get_ssrc (GstRTPBuffer * rtp)
 {
-  return g_ntohl (GST_RTP_HEADER_SSRC (rtp->map[0].data));
+  return g_ntohl (GST_RTP_HEADER_SSRC (rtp->data[0]));
 }
 
 /**
@@ -742,7 +807,7 @@ gst_rtp_buffer_get_ssrc (GstRTPBuffer * rtp)
 void
 gst_rtp_buffer_set_ssrc (GstRTPBuffer * rtp, guint32 ssrc)
 {
-  GST_RTP_HEADER_SSRC (rtp->map[0].data) = g_htonl (ssrc);
+  GST_RTP_HEADER_SSRC (rtp->data[0]) = g_htonl (ssrc);
 }
 
 /**
@@ -756,7 +821,7 @@ gst_rtp_buffer_set_ssrc (GstRTPBuffer * rtp, guint32 ssrc)
 guint8
 gst_rtp_buffer_get_csrc_count (GstRTPBuffer * rtp)
 {
-  return GST_RTP_HEADER_CSRC_COUNT (rtp->map[0].data);
+  return GST_RTP_HEADER_CSRC_COUNT (rtp->data[0]);
 }
 
 /**
@@ -773,7 +838,7 @@ gst_rtp_buffer_get_csrc (GstRTPBuffer * rtp, guint8 idx)
 {
   guint8 *data;
 
-  data = rtp->map[0].data;
+  data = rtp->data[0];
 
   g_return_val_if_fail (idx < GST_RTP_HEADER_CSRC_COUNT (data), 0);
 
@@ -793,7 +858,7 @@ gst_rtp_buffer_set_csrc (GstRTPBuffer * rtp, guint8 idx, guint32 csrc)
 {
   guint8 *data;
 
-  data = rtp->map[0].data;
+  data = rtp->data[0];
 
   g_return_if_fail (idx < GST_RTP_HEADER_CSRC_COUNT (data));
 
@@ -811,7 +876,7 @@ gst_rtp_buffer_set_csrc (GstRTPBuffer * rtp, guint8 idx, guint32 csrc)
 gboolean
 gst_rtp_buffer_get_marker (GstRTPBuffer * rtp)
 {
-  return GST_RTP_HEADER_MARKER (rtp->map[0].data);
+  return GST_RTP_HEADER_MARKER (rtp->data[0]);
 }
 
 /**
@@ -824,7 +889,7 @@ gst_rtp_buffer_get_marker (GstRTPBuffer * rtp)
 void
 gst_rtp_buffer_set_marker (GstRTPBuffer * rtp, gboolean marker)
 {
-  GST_RTP_HEADER_MARKER (rtp->map[0].data) = marker;
+  GST_RTP_HEADER_MARKER (rtp->data[0]) = marker;
 }
 
 /**
@@ -838,7 +903,7 @@ gst_rtp_buffer_set_marker (GstRTPBuffer * rtp, gboolean marker)
 guint8
 gst_rtp_buffer_get_payload_type (GstRTPBuffer * rtp)
 {
-  return GST_RTP_HEADER_PAYLOAD_TYPE (rtp->map[0].data);
+  return GST_RTP_HEADER_PAYLOAD_TYPE (rtp->data[0]);
 }
 
 /**
@@ -853,7 +918,7 @@ gst_rtp_buffer_set_payload_type (GstRTPBuffer * rtp, guint8 payload_type)
 {
   g_return_if_fail (payload_type < 0x80);
 
-  GST_RTP_HEADER_PAYLOAD_TYPE (rtp->map[0].data) = payload_type;
+  GST_RTP_HEADER_PAYLOAD_TYPE (rtp->data[0]) = payload_type;
 }
 
 /**
@@ -867,7 +932,7 @@ gst_rtp_buffer_set_payload_type (GstRTPBuffer * rtp, guint8 payload_type)
 guint16
 gst_rtp_buffer_get_seq (GstRTPBuffer * rtp)
 {
-  return g_ntohs (GST_RTP_HEADER_SEQ (rtp->map[0].data));
+  return g_ntohs (GST_RTP_HEADER_SEQ (rtp->data[0]));
 }
 
 /**
@@ -880,7 +945,7 @@ gst_rtp_buffer_get_seq (GstRTPBuffer * rtp)
 void
 gst_rtp_buffer_set_seq (GstRTPBuffer * rtp, guint16 seq)
 {
-  GST_RTP_HEADER_SEQ (rtp->map[0].data) = g_htons (seq);
+  GST_RTP_HEADER_SEQ (rtp->data[0]) = g_htons (seq);
 }
 
 /**
@@ -894,7 +959,7 @@ gst_rtp_buffer_set_seq (GstRTPBuffer * rtp, guint16 seq)
 guint32
 gst_rtp_buffer_get_timestamp (GstRTPBuffer * rtp)
 {
-  return g_ntohl (GST_RTP_HEADER_TIMESTAMP (rtp->map[0].data));
+  return g_ntohl (GST_RTP_HEADER_TIMESTAMP (rtp->data[0]));
 }
 
 /**
@@ -907,7 +972,7 @@ gst_rtp_buffer_get_timestamp (GstRTPBuffer * rtp)
 void
 gst_rtp_buffer_set_timestamp (GstRTPBuffer * rtp, guint32 timestamp)
 {
-  GST_RTP_HEADER_TIMESTAMP (rtp->map[0].data) = g_htonl (timestamp);
+  GST_RTP_HEADER_TIMESTAMP (rtp->data[0]) = g_htonl (timestamp);
 }
 
 
@@ -922,8 +987,6 @@ gst_rtp_buffer_set_timestamp (GstRTPBuffer * rtp, guint32 timestamp)
  * If @len is -1 the total payload starting from @offset if subbuffered.
  *
  * Returns: A new buffer with the specified data of the payload.
- *
- * Since: 0.10.10
  */
 GstBuffer *
 gst_rtp_buffer_get_payload_subbuffer (GstRTPBuffer * rtp, guint offset,
@@ -982,18 +1045,8 @@ gst_rtp_buffer_get_payload_buffer (GstRTPBuffer * rtp)
 guint
 gst_rtp_buffer_get_payload_len (GstRTPBuffer * rtp)
 {
-  guint len, size;
-  guint8 *data;
-
-  size = rtp->map[0].size;
-  data = rtp->map[0].data;
-
-  len = size - gst_rtp_buffer_get_header_len (rtp);
-
-  if (GST_RTP_HEADER_PADDING (data))
-    len -= data[size - 1];
-
-  return len;
+  return gst_buffer_get_size (rtp->buffer) - gst_rtp_buffer_get_header_len (rtp)
+      - rtp->size[3];
 }
 
 /**
@@ -1003,12 +1056,33 @@ gst_rtp_buffer_get_payload_len (GstRTPBuffer * rtp)
  * Get a pointer to the payload data in @buffer. This pointer is valid as long
  * as a reference to @buffer is held.
  *
- * Returns: A pointer to the payload data in @buffer.
+ * Returns: (array) (element-type guint8) (transfer none): A pointer
+ * to the payload data in @buffer.
  */
 gpointer
 gst_rtp_buffer_get_payload (GstRTPBuffer * rtp)
 {
-  return rtp->map[0].data + gst_rtp_buffer_get_header_len (rtp);
+  guint hlen, plen;
+  guint idx, length;
+  gsize skip;
+
+  if (rtp->data[2])
+    return rtp->data[2];
+
+  hlen = gst_rtp_buffer_get_header_len (rtp);
+  plen = gst_buffer_get_size (rtp->buffer) - hlen - rtp->size[3];
+
+  if (!gst_buffer_find_memory (rtp->buffer, hlen, plen, &idx, &length, &skip))
+    return NULL;
+
+  if (!gst_buffer_map_range (rtp->buffer, idx, length, &rtp->map[2],
+          rtp->map[0].flags))
+    return NULL;
+
+  rtp->data[2] = rtp->map[2].data + skip;
+  rtp->size[2] = plen;
+
+  return rtp->data[2];
 }
 
 /**
@@ -1019,8 +1093,6 @@ gst_rtp_buffer_get_payload (GstRTPBuffer * rtp)
  *
  * Returns: the default clock rate or -1 if the payload type is not static or
  * the clock-rate is undefined.
- *
- * Since: 0.10.13
  */
 guint32
 gst_rtp_buffer_default_clock_rate (guint8 payload_type)
@@ -1050,8 +1122,6 @@ gst_rtp_buffer_default_clock_rate (guint8 payload_type)
  *
  * Returns: a negative value if @seqnum1 is bigger than @seqnum2, 0 if they
  * are equal or a positive value if @seqnum1 is smaller than @segnum2.
- *
- * Since: 0.10.15
  */
 gint
 gst_rtp_buffer_compare_seqnum (guint16 seqnum1, guint16 seqnum2)
@@ -1071,8 +1141,6 @@ gst_rtp_buffer_compare_seqnum (guint16 seqnum1, guint16 seqnum2)
  * value even in the case where there is a timestamp wraparound.
  *
  * Returns: The extended timestamp of @timestamp.
- *
- * Since: 0.10.15
  */
 guint64
 gst_rtp_buffer_ext_timestamp (guint64 * exttimestamp, guint32 timestamp)
@@ -1111,15 +1179,14 @@ gst_rtp_buffer_ext_timestamp (guint64 * exttimestamp, guint32 timestamp)
  * @rtp: the RTP packet
  * @id: The ID of the header extension to be read (between 1 and 14).
  * @nth: Read the nth extension packet with the requested ID
- * @data: location for data
- * @size: the size of the data in bytes
+ * @data: (out) (array length=size) (element-type guint8) (transfer none):
+ *   location for data
+ * @size: (out): the size of the data in bytes
  *
  * Parses RFC 5285 style header extensions with a one byte header. It will
  * return the nth extension with the requested id.
  *
  * Returns: TRUE if @buffer had the requested header extension
- *
- * Since: 0.10.31
  */
 
 gboolean
@@ -1188,18 +1255,17 @@ gst_rtp_buffer_get_extension_onebyte_header (GstRTPBuffer * rtp, guint8 id,
 /**
  * gst_rtp_buffer_get_extension_twobytes_header:
  * @rtp: the RTP packet
- * @appbits: Application specific bits
+ * @appbits: (out): Application specific bits
  * @id: The ID of the header extension to be read (between 1 and 14).
  * @nth: Read the nth extension packet with the requested ID
- * @data: location for data
- * @size: the size of the data in bytes
+ * @data: (out) (array length=size) (element-type guint8) (transfer none):
+ *   location for data
+ * @size: (out): the size of the data in bytes
  *
  * Parses RFC 5285 style header extensions with a two bytes header. It will
  * return the nth extension with the requested id.
  *
  * Returns: TRUE if @buffer had the requested header extension
- *
- * Since: 0.10.31
  */
 
 gboolean
@@ -1303,7 +1369,7 @@ get_onebyte_header_end_offset (guint8 * pdata, guint wordlen)
  * gst_rtp_buffer_add_extension_onebyte_header:
  * @rtp: the RTP packet
  * @id: The ID of the header extension (between 1 and 14).
- * @data: location for data
+ * @data: (array length=size) (element-type guint8): location for data
  * @size: the size of the data in bytes
  *
  * Adds a RFC 5285 header extension with a one byte header to the end of the
@@ -1315,8 +1381,6 @@ get_onebyte_header_end_offset (guint8 * pdata, guint wordlen)
  * gst_rtp_buffer_add_extension_twobytes_header()
  *
  * Returns: %TRUE if header extension could be added
- *
- * Since: 0.10.31
  */
 
 gboolean
@@ -1327,6 +1391,7 @@ gst_rtp_buffer_add_extension_onebyte_header (GstRTPBuffer * rtp, guint8 id,
   guint8 *pdata = 0;
   guint wordlen;
   gboolean has_bit;
+  guint extlen, offset = 0;
 
   g_return_val_if_fail (id > 0 && id < 15, FALSE);
   g_return_val_if_fail (size >= 1 && size <= 16, FALSE);
@@ -1336,51 +1401,30 @@ gst_rtp_buffer_add_extension_onebyte_header (GstRTPBuffer * rtp, guint8 id,
       (gpointer) & pdata, &wordlen);
 
   if (has_bit) {
-    gulong offset = 0;
-    guint8 *nextext;
-    guint extlen;
-
     if (bits != 0xBEDE)
       return FALSE;
 
     offset = get_onebyte_header_end_offset (pdata, wordlen);
     if (offset == 0)
       return FALSE;
-
-    nextext = pdata + offset;
-    offset = nextext - rtp->map[0].data;
-
-    /* Don't add extra header if there isn't enough space */
-    if (rtp->map[0].size < offset + size + 1)
-      return FALSE;
-
-    nextext[0] = (id << 4) | (0x0F & (size - 1));
-    memcpy (nextext + 1, data, size);
-
-    extlen = nextext - pdata + size + 1;
-    if (extlen % 4) {
-      wordlen = extlen / 4 + 1;
-      memset (nextext + size + 1, 0, 4 - extlen % 4);
-    } else {
-      wordlen = extlen / 4;
-    }
-
-    gst_rtp_buffer_set_extension_data (rtp, 0xBEDE, wordlen);
-  } else {
-    wordlen = (size + 1) / 4 + (((size + 1) % 4) ? 1 : 0);
-
-    gst_rtp_buffer_set_extension_data (rtp, 0xBEDE, wordlen);
-
-    gst_rtp_buffer_get_extension_data (rtp, &bits,
-        (gpointer) & pdata, &wordlen);
-
-    pdata[0] = (id << 4) | (0x0F & (size - 1));
-    memcpy (pdata + 1, data, size);
-
-    if ((size + 1) % 4)
-      memset (pdata + size + 1, 0, 4 - ((size + 1) % 4));
   }
 
+  /* the required size of the new extension data */
+  extlen = offset + size + 1;
+  /* calculate amount of words */
+  wordlen = extlen / 4 + ((extlen % 4) ? 1 : 0);
+
+  gst_rtp_buffer_set_extension_data (rtp, 0xBEDE, wordlen);
+  gst_rtp_buffer_get_extension_data (rtp, &bits, (gpointer) & pdata, &wordlen);
+
+  pdata += offset;
+
+  pdata[0] = (id << 4) | (0x0F & (size - 1));
+  memcpy (pdata + 1, data, size);
+
+  if (extlen % 4)
+    memset (pdata + 1 + size, 0, 4 - (extlen % 4));
+
   return TRUE;
 }
 
@@ -1424,7 +1468,7 @@ get_twobytes_header_end_offset (guint8 * pdata, guint wordlen)
  * @rtp: the RTP packet
  * @appbits: Application specific bits
  * @id: The ID of the header extension
- * @data: location for data
+ * @data: (array length=size) (element-type guint8): location for data
  * @size: the size of the data in bytes
  *
  * Adds a RFC 5285 header extension with a two bytes header to the end of the
@@ -1436,8 +1480,6 @@ get_twobytes_header_end_offset (guint8 * pdata, guint wordlen)
  * gst_rtp_buffer_add_extension_onebyte_header()
  *
  * Returns: %TRUE if header extension could be added
- *
- * Since: 0.10.31
  */
 
 gboolean
@@ -1448,6 +1490,8 @@ gst_rtp_buffer_add_extension_twobytes_header (GstRTPBuffer * rtp,
   guint8 *pdata = 0;
   guint wordlen;
   gboolean has_bit;
+  gulong offset = 0;
+  guint extlen;
 
   g_return_val_if_fail ((appbits & 0xF0) == 0, FALSE);
   g_return_val_if_fail (size < 256, FALSE);
@@ -1457,52 +1501,30 @@ gst_rtp_buffer_add_extension_twobytes_header (GstRTPBuffer * rtp,
       (gpointer) & pdata, &wordlen);
 
   if (has_bit) {
-    gulong offset = 0;
-    guint8 *nextext;
-    guint extlen;
-
     if (bits != ((0x100 << 4) | (appbits & 0x0f)))
       return FALSE;
 
     offset = get_twobytes_header_end_offset (pdata, wordlen);
-
-    nextext = pdata + offset;
-
-    offset = nextext - rtp->map[0].data;
-
-    /* Don't add extra header if there isn't enough space */
-    if (rtp->map[0].size < offset + size + 2)
+    if (offset == 0)
       return FALSE;
-
-    nextext[0] = id;
-    nextext[1] = size;
-    memcpy (nextext + 2, data, size);
-
-    extlen = nextext - pdata + size + 2;
-    if (extlen % 4) {
-      wordlen = extlen / 4 + 1;
-      memset (nextext + size + 2, 0, 4 - extlen % 4);
-    } else {
-      wordlen = extlen / 4;
-    }
-
-    gst_rtp_buffer_set_extension_data (rtp, (0x100 << 4) | (appbits & 0x0F),
-        wordlen);
-  } else {
-    wordlen = (size + 2) / 4 + (((size + 2) % 4) ? 1 : 0);
-
-    gst_rtp_buffer_set_extension_data (rtp, (0x100 << 4) | (appbits & 0x0F),
-        wordlen);
-
-    gst_rtp_buffer_get_extension_data (rtp, &bits,
-        (gpointer) & pdata, &wordlen);
-
-    pdata[0] = id;
-    pdata[1] = size;
-    memcpy (pdata + 2, data, size);
-    if ((size + 2) % 4)
-      memset (pdata + size + 2, 0, 4 - ((size + 2) % 4));
   }
 
+  /* the required size of the new extension data */
+  extlen = offset + size + 2;
+  /* calculate amount of words */
+  wordlen = extlen / 4 + ((extlen % 4) ? 1 : 0);
+
+  gst_rtp_buffer_set_extension_data (rtp, (0x100 << 4) | (appbits & 0x0F),
+      wordlen);
+  gst_rtp_buffer_get_extension_data (rtp, &bits, (gpointer) & pdata, &wordlen);
+
+  pdata += offset;
+
+  pdata[0] = id;
+  pdata[1] = size;
+  memcpy (pdata + 2, data, size);
+  if (extlen % 4)
+    memset (pdata + 2 + size, 0, 4 - (extlen % 4));
+
   return TRUE;
 }
